import { VisibilityRulesService } from './visibility-rules/visibility-rules.service';
import { DynamicFormSectionComponent } from './form/dynamic-form-section/dynamic-form-section';
import { DynamicFormCompositeFieldComponent } from './form/dynamic-form-composite-field/dynamic-form-composite-field';
import { BrowserModule } from '@angular/platform-browser';
import { NgModule, forwardRef } from '@angular/core';
import { FormsModule, ReactiveFormsModule, NG_VALUE_ACCESSOR } from '@angular/forms';
import { HttpClientModule } from '@angular/common/http';
import { HttpModule } from '@angular/http';
import { RouterModule, Routes, Router } from '@angular/router';
//import {DataTableModule } from 'angular-4-data-table-bootstrap-4';
import { DataTableModule } from "angular2-datatable";

//import { OrganisationTableFilterPipe } from './pipes/organisation-table-filter.pipe';

import { AppComponent } from './app.component';
import { RestBase } from './services/rest-base';
import { DynamicFormComponent } from './form/dynamic-form.component';
import { DynamicFormFieldComponent } from './form/fields/dynamic-form-field.component';
import { ServerService } from './services/server.service';
import { GlobalVariables } from './services/global-variables.service';
import { TokenService, TokenProvider } from './services/login/token.service';
import { LocalStorageService } from 'ngx-webstorage';
import { dataModelBuilder } from './services/dataModelBuilder.service';
import { DynamicFormGroupComponent } from './form/dynamic-form-group/dynamic-form-group.component';
import { AppRoutingModule }        from './app-routing.module';
import { AuthGuard } from './guards/auth.guard';
import { PageNotFoundComponent }   from './not-found.component';
import { HomepageComponent }    from './homepage/homepage.component';
import { TocComponent } from './form/tableOfContents/toc.component';
import { ConfirmationComponent } from './widgets/confirmation/confirmation.component';


import { PaginationService } from './services/pagination.service';
import { EestoreService } from './services/eestore.service';
import { GlobalInterceptor } from './services/interceptor';


import { TabModule } from './tabs/tab.module';
import { AngularDraggableModule } from 'angular2-draggable';

import { NativeLoginService } from './services/login/native-login.service';
import { GooggleSignInComponent } from './login/googgle-sign-in/googgle-sign-in.component';
import { MainSignInComponent } from './login/main-sign-in/main-sign-in.component';
import { NguiAutoCompleteModule } from '@ngui/auto-complete';

import { HTTP_INTERCEPTORS } from '@angular/common/http';

import {BreadcrumbModule,MenuItem} from 'primeng/primeng';

import { CommonModule }   from '@angular/common';

import { DatasetTableFilterPipe } from './pipes/dataset-table-filter.pipe';
import { DatasetStatusFilterPipe } from './pipes/dataset-status-filter.pipe';
import { DatasetsComponent }  from './datasets/dataset.component';


import { DmpComponent }  from './dmps/dmp.component';
import { DmpTableFilterPipe } from './pipes/dmp-table-filter.pipe';
import { DmpStatusFilterPipe } from './pipes/dmp-status-filter.pipe';
import { DmpVersionFilterPipe } from './pipes/dmp-version-filter.pipe';


import { ProjectTableFilterPipe } from './pipes/project-table-filter.pipe';
import { ProjectsComponent }  from './projects/projects.component';
import { ProjectDetailComponent } from './projects/project.detail';
import { ModalComponent } from './modal/modal.component';
import { NgDatepickerModule } from 'ng2-datepicker';

import { StatusToString } from './pipes/various/status-to-string';
import { SidebarModule } from 'ng-sidebar';
import { AutocompleteRemoteComponent } from './form/fields/autocomplete-remote/autocomplete-remote.component';

import { Ng4LoadingSpinnerModule } from 'ng4-loading-spinner';
import { BreadcrumbComponent } from './widgets/breadcrumb/breadcrumb.component';
import { DmpDetailedComponent } from './viewers/dmp-detailed/dmp-detailed.component';
import { ProjectDetailedComponent } from './viewers/project-detailed/project-detailed.component';




@NgModule({
  declarations: [
    AppComponent,
    DynamicFormComponent,
    DynamicFormFieldComponent,
    DynamicFormGroupComponent,
    DynamicFormCompositeFieldComponent,
    TocComponent,
    GooggleSignInComponent,
    MainSignInComponent,
    PageNotFoundComponent,
    HomepageComponent,
    ModalComponent,
    ProjectDetailComponent,
    ProjectsComponent,
    DmpComponent,
    DatasetsComponent,
    ConfirmationComponent,
    AutocompleteRemoteComponent,
    DynamicFormSectionComponent,
    ProjectTableFilterPipe, 
    DmpVersionFilterPipe, 
    DmpStatusFilterPipe, 
    DmpTableFilterPipe,
    DatasetTableFilterPipe, 
    DatasetStatusFilterPipe, 
    StatusToString, 
    BreadcrumbComponent, DmpDetailedComponent, ProjectDetailedComponent
  ],
  imports: [
    BrowserModule,
    ReactiveFormsModule,
    FormsModule,
    HttpModule,
    HttpClientModule,
    TabModule,
    CommonModule,
    AppRoutingModule,
    AngularDraggableModule,
    DataTableModule,
    NgDatepickerModule,
    Ng4LoadingSpinnerModule,
    NguiAutoCompleteModule,
    BreadcrumbModule,
    SidebarModule.forRoot()
    
  ],
  providers: [{
                  provide: HTTP_INTERCEPTORS,
                  useClass: GlobalInterceptor,
                  multi: true,
              },
              ServerService, dataModelBuilder,VisibilityRulesService, GlobalVariables, AuthGuard, PaginationService, TokenService, LocalStorageService, RestBase, EestoreService,NativeLoginService
            ],
  bootstrap: [AppComponent]
})
export class AppModule {
  constructor() {
  }
}
