import { Component, OnInit } from '@angular/core';
import { Router, CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, ActivatedRoute, NavigationEnd } from '@angular/router';
import {BreadcrumbModule,MenuItem} from 'primeng/primeng';

@Component({
  selector: 'breadcrumb',
  templateUrl: './breadcrumb.component.html',
  styleUrls: ['./breadcrumb.component.css']
})
export class BreadcrumbComponent implements OnInit {


  breadcrumbHome: MenuItem = {icon: 'fa fa-home', routerLink: "/welcome"};
  breadcrumbData: MenuItem[] = new Array<MenuItem>();

  ngOnInit() {
    
  }

  constructor(private router: Router, private route: ActivatedRoute) {
    router.events.subscribe(
      event =>{
        //console.log("Router event captured")
        //console.log(event)
        if(event instanceof NavigationEnd){
          //console.log(event.urlAfterRedirects);
          //console.log(this.route);
          this.route.children.forEach( child => {
            let guessed = this.guessMenuItemFromActivatedRoute(child, event);
            this.adaptBreadcrumbByMenuItem(guessed.menuItem, guessed.isBaseComponent);
          })

        }
      });

  }


  guessMenuItemFromActivatedRoute(activatedRoute : any, event : NavigationEnd) { //it's important to leave it as "any"

    let menuItem : MenuItem = null;
    let isBaseComponent : boolean = false;

    let componentName = activatedRoute.component.name; 
    
    let params = activatedRoute.queryParams.getValue();
    let url = event.urlAfterRedirects.split("?")[0]; 
    let label = null;

    if(componentName == "ProjectsComponent") {
      label = "Projects";
      isBaseComponent = true;
    }
    if(componentName == "DmpComponent"){
      label = "My Data Management Plans";
      isBaseComponent = true;
    }

    if(componentName == "DatasetsComponent"){
      label = "Datasets of DMP '"+params["label"]+"'";
      isBaseComponent = false;
    }

    if(componentName == "DynamicFormComponent"){
      label = "Form of dataset '"+params["label"]+"'";
      isBaseComponent = false;
    }

    if(componentName == "DmpDetailedComponent"){
      label = "Details of DMP '"+params["label"]+"'";
      isBaseComponent = false;
    }

    if(componentName == "ProjectDetailedComponent"){
      label = "Details of Project '"+params["label"]+"'";
      isBaseComponent = false;
    }

    
    if(label != null)
      menuItem = {"label": label, "routerLink": url, "queryParams" : params };

    console.log("COMPONENT NAME="+componentName);

    return {"menuItem": menuItem, "isBaseComponent": isBaseComponent};
  }


  adaptBreadcrumbByMenuItem(menuItem : MenuItem, isBaseComponent : boolean){

    if(menuItem==null){
      this.breadcrumbData.length = 0;
      return;
    }
    if(isBaseComponent){
      this.breadcrumbData.length = 0;
    }

    let breadcrumbDataNew: MenuItem[] = new Array<MenuItem>();
    for(var i=0; i<this.breadcrumbData.length;i++){
      if(this.breadcrumbData[i].label == menuItem.label)
        break;  
      else
        breadcrumbDataNew.push(this.breadcrumbData[i]);
    }
    this.breadcrumbData = breadcrumbDataNew;

    this.breadcrumbData.push(menuItem);

  }




}
